@@ -1,34 +1,64 @@
 # -*- coding:utf-8 -*-
 
+import os
+import re
+import textwrap
 import random
-from typing import Optional, Union
+import logging
+from typing import Dict, List, Optional, Tuple, Union
 
-from ..common import (
-    Fx,
-    Color,
-    Emoji,
-    render_str,
-    run_cmd,
-    confirm,
-    similar_command,
-    shorten,
-)
+from ..common.utils import run_cmd, confirm, similar_command, traceback_info
+from ..common.singleton import Singleton
+from ..render import get_console
+from ..render.str_utils import shorten
 from .cmds import Git_Cmds, CommandType
 
+Log = logging.getLogger(__name__)
 
-class CmdProcessor(object):
+
+def get_extra_cmds(name: str, path: str) -> Dict:
+    """Get custom cmds.
+
+    Load the `extra_cmds.py` file under PIGIT HOME, check whether `extra_cmds`
+    exists, and return it. If not have, return a empty dict.
+
+    Returns:
+        (dict[str,str]): extra cmds dict.
+    """
+    import importlib.util
+
+    extra_cmds = {}
+
+    if os.path.isfile(path):
+        try:
+            # load a module form localtion.
+            spec = importlib.util.spec_from_file_location(name, path)
+            module = importlib.util.module_from_spec(spec)
+            spec.loader.exec_module(module)
+        except Exception:
+            Log.error(traceback_info(f"Can't load file '{path}'."))
+        else:
+            try:
+                extra_cmds = module.extra_cmds  # type: ignore
+            except AttributeError:
+                Log.error("Can't found dict name is 'extra_cmds'.")
+
+    return extra_cmds
+
+
+class CmdProcessor(object, metaclass=Singleton):
     """Git short command processor."""
 
     def __init__(
         self,
         extra_cmds: Optional[dict] = None,
-        use_recommend: bool = True,
+        command_prompt: bool = True,
         show_original: bool = True,
         **kwargs,
     ) -> None:
         super(CmdProcessor, self).__init__()
 
-        self.use_recommend = use_recommend
+        self.use_recommend = command_prompt
         self.show_original = show_original
 
         self.cmds = Git_Cmds
@@ -52,29 +82,23 @@ class CmdProcessor(object):
             (str): color command string.
         """
 
-        command_list = command.split(" ")
-        color_command = render_str(
-            f"b`{command_list.pop(0)}`<ok> b`{command_list.pop(0)}`<goldenrod> "
-        )
+        handle = re.match(r"(\w+)\s+(\w+)", command)
+        prop = handle.group(1)
+        cmd = handle.group(2)
+        next_position = handle.span()[1]
 
-        less_len = len(command_list)
-        idx = 0
-        while idx < less_len:
-            if command_list[idx].startswith("-"):
-                temp = []
-                while idx < less_len and command_list[idx].startswith("-"):
-                    temp.append(command_list[idx])
-                    idx += 1
-                color_command += render_str(f"`{' '.join(temp)}`<sky_blue> ")
-            else:
-                while idx < less_len and not command_list[idx].startswith("-"):
-                    color_command += command_list[idx] + " "
-                    idx += 1
+        color_command = f"b`{prop}`<ok> b`{cmd}`<goldenrod> "
+
+        arguments = re.findall(
+            r"\s+(-?-?[^\s]+)(=?([\w-]+:)?(\".*?\"|\'.*?\'))?", command[next_position:]
+        )
+        for arg_handle, value, _, _ in arguments:
+            color_command += f"`{arg_handle}`<sky_blue>{value} "
 
         return color_command
 
     def process_command(
-        self, command_: str, args: Optional[Union[list, tuple]] = None
+        self, command_: str, args: Optional[Union[List, Tuple]] = None
     ) -> None:
         """Process command and arguments.
 
@@ -87,20 +111,18 @@ class CmdProcessor(object):
             SystemExit: short command not right.
         """
 
-        option: Optional[dict] = self.cmds.get(command_, None)
+        option: Optional[Dict] = self.cmds.get(command_, None)
 
         # Invalid, if need suggest.
         if option is None:
-            print(
-                render_str(
-                    "Don't support this command, please try `g --show-commands`<gold>"
-                )
+            get_console().echo(
+                "Don't support this command, please try `g --show-commands`<gold>"
             )
 
             if self.use_recommend:  # check config.
                 predicted_command = similar_command(command_, self.cmds.keys())
                 if confirm(
-                    render_str(
+                    get_console().render_str(
                         f":thinking: The wanted command is `{predicted_command}`<ok> ?[y/n]:"
                     )
                 ):
@@ -111,42 +133,37 @@ class CmdProcessor(object):
         command = option.get("command", None)
         # Has no command can be executed.
         if not command:
-            print(
-                render_str(
-                    "`Invalid custom short command, nothing can to exec.`<error>"
-                )
+            get_console().echo(
+                "`Invalid custom short command, nothing can to exec.`<error>"
             )
             return None
 
-        if not option.get("has_arguments", False):
-            if args:
-                print(
-                    render_str(
-                        f"`The command does not accept parameters. Discard {args}.`<error>"
-                    )
-                )
-                args = []
+        if not option.get("has_arguments", False) and args:
+            get_console().echo(
+                f"`The command does not accept parameters. Discard {args}.`<error>"
+            )
+            args = []
 
         _type = option.get("type", "command")
         if _type == "func":
             try:
                 command(args)
             except Exception as e:
-                print(render_str(f"`{e}`<error>"))
+                get_console().echo(f"`{e}`<error>")
         else:  # is command.
             if args:
                 args_str = " ".join(args)
                 command = " ".join([command, args_str])
             if self.show_original:
-                print(
-                    "{0}  {1}".format(Emoji.rainbow, self.color_command(command)),
-                )
+                get_console().echo(f":rainbow:  {self.color_command(command)}")
             run_cmd(command)
 
     ################################
     # Print command help message.
     ################################
-    def _generate_help_by_key(self, _key: str, use_color: bool = True) -> str:
+    def _generate_help_by_key(
+        self, _key: str, use_color: bool = True, max_width=90
+    ) -> str:
         """Generate one help by given key.
 
         Args:
@@ -157,47 +174,37 @@ class CmdProcessor(object):
             (str): Help message of one command.
         """
 
-        _msg: str = "    {key_color}{:<9}{reset}{}{command_color}{}{reset}"
-        if use_color:
-            _key_color = Color.by_name("ok")
-            _command_color = Color.by_name("gold")
-        else:
-            _key_color = _command_color = ""
+        help_position = 15
+        msg_max_width = max_width - help_position
 
         # Get help message and command.
-        _help: str = self.cmds[_key].get("help", "")
-        try:
-            # must have key `command`.
-            _command = self.cmds[_key]["command"]
-        except:
-            raise ValueError("The `command` key can not be empty.")
+        _help: str = self.cmds[_key].get("help", "").strip()
+        if _help:
+            _help = textwrap.wrap(_help, msg_max_width)
+            help_msg = _help[0] + "\n"
+            for line in _help[1:]:
+                help_msg += "%*s%s\n" % (help_position, "", line)
+        else:
+            help_msg = ""
 
-        # Process help.
-        _help = _help + "\n" if _help else ""
-
-        # Process command.
+        _command = self.cmds[_key].get("command", "ERROR: empty command.")
         if callable(_command):
-            _command = "Callable: %s" % _command.__name__
+            _command = f"Func: {_command.__name__}"
 
-        _command = shorten(_command, 70, placeholder="...")
-        _command = " " * 13 + _command if _help else _command
+        _command = shorten(_command, msg_max_width, placeholder="...")
+        command_msg = "%*s%s" % (help_position, "", _command) if help_msg else _command
 
-        # Splicing and return.
-        return _msg.format(
-            _key,
-            _help,
-            _command,
-            key_color=_key_color,
-            command_color=_command_color,
-            reset=Fx.reset,
-        )
+        if use_color:
+            return f"  `{_key:<13}`<ok>{help_msg}`{command_msg}`<gold>"
+        else:
+            return f"  {_key:<12} {_help}{command_msg}"
 
     def command_help(self) -> None:
         """Print help message."""
         print("These are short commands that can replace git operations:")
         for key in self.cmds.keys():
             msg = self._generate_help_by_key(key)
-            print(msg)
+            get_console().echo(msg)
 
     def command_help_by_type(self, command_type: str) -> None:
         """Print a part of help message.
@@ -215,11 +222,9 @@ class CmdProcessor(object):
 
         # Checking the type whether right.
         if command_type not in CommandType.__members__:
-            print(
-                render_str(
-                    "`There is no such type.`<error>\n"
-                    "Please use `git --types`<ok> to view the supported types."
-                )
+            get_console().echo(
+                "`There is no such type.`<error>\n"
+                "Please use `git --types`<ok> to view the supported types."
             )
 
             if self.use_recommend:
@@ -227,7 +232,7 @@ class CmdProcessor(object):
                     command_type, CommandType.__members__.keys()
                 )
                 if confirm(
-                    render_str(
+                    get_console().render_str(
                         f":thinking: The wanted type is `{predicted_type}`<ok> ?[y/n]:"
                     )
                 ):
@@ -241,21 +246,20 @@ class CmdProcessor(object):
             # Prevent the `belong` attribute from being set in the custom command.
             if isinstance(belong, CommandType) and belong.value == command_type:
                 msg = self._generate_help_by_key(k)
-                print(msg)
+                get_console().echo(msg)
 
     @classmethod
     def type_help(cls) -> None:
         """Print all command types with random color."""
+        res = []
+
         for member in CommandType:
-            print(
-                "{0}{1}  ".format(
-                    Color.fg(
-                        random.randint(70, 255),
-                        random.randint(70, 255),
-                        random.randint(70, 255),
-                    ),
-                    member.value,
-                ),
-                end="",
+            color_str = "#{:02X}{:02X}{:02X}".format(
+                random.randint(70, 255),
+                random.randint(70, 255),
+                random.randint(70, 255),
             )
-        print(Fx.reset)
+
+            res.append(f"`{member.value}`<{color_str}>")
+
+        get_console().echo(" ".join(res))
