@@ -1,9 +1,8 @@
 # -*- coding:utf-8 -*-
 
-import os
-import re
+from typing import Dict, Optional
+import os, re
 import logging
-from typing import Optional
 
 
 Log = logging.getLogger(__name__)
@@ -20,13 +19,13 @@ class ShellCompletion(object):
 
     _SHELL: str  # shell name.
     _INJECT_PATH: str  # script inject path.
-    _TEMPLATE: str  # script tempelate string.
+    _template_source: str  # script tempelate string.
 
     def __init__(
         self,
-        prop: str,
-        complete_vars: dict,
-        script_dir: str,
+        prog_name: Optional[str] = None,
+        complete_vars: Dict = None,
+        script_dir: Optional[str] = None,
         script_name: Optional[str] = None,
         inject_path: Optional[str] = None,
     ) -> None:
@@ -48,16 +47,61 @@ class ShellCompletion(object):
         """
         super(ShellCompletion, self).__init__()
 
-        self.prop = prop
-
         if not isinstance(complete_vars, dict):
             raise TypeError("complete_var muse be dict.")
         self.complete_vars = complete_vars
 
-        self.script_dir = script_dir
-        self.script_name = script_name or "{0}_{1}_comp".format(self._SHELL, self.prop)
+        if prog_name is not None:
+            self.prog_name = prog_name
+        elif inner_prog_name := complete_vars.get("prog"):
+            self.prog_name = inner_prog_name
+        else:
+            raise ShellCompletionError("Can't get prog name anywhere.") from None
+
+        self.script_dir = script_dir or "."
+        self.script_name = script_name or "{0}_{1}_comp".format(
+            self.prog_name, self._SHELL
+        )
         self.inject_path = inject_path or self._INJECT_PATH
 
+    @property
+    def func_name(self) -> str:
+        """The name of the shell function defined by the completion
+        script.
+        """
+
+        safe_name = re.sub(r"\W*", "", self.prog_name.replace("-", "_"), re.ASCII)
+        return f"_{safe_name}_completion"
+
+    def _parse(self, args: Dict):
+        _arguments = []
+        _positions = []
+        _sub_opts = {}
+
+        for name, prop in args.items():
+            if name == "-groups":
+                for g_name, g_p in prop.items():
+                    a, p, s = self._parse(g_p["args"])
+                    _arguments.extend(a)
+                    _positions.extend(p)
+            elif name == "set_defaults":
+                # Special need be ignore.
+                pass
+            elif "args" in prop:
+                a, p, s = self._parse(prop["args"])
+                _sub_opts[name] = {
+                    "_arguments": a,
+                    "_positions": p,
+                    "_sub_opts": s,
+                    "help": prop.get("help", "_"),
+                }
+            elif name.startswith("-"):
+                _arguments.append((name, prop.get("help", "_")))
+            else:
+                _positions.append((name, prop.get("help", "_")))
+
+        return _arguments, _positions, _sub_opts
+
     def generate(self):
         """Generate script content.
 
@@ -79,13 +123,12 @@ class ShellCompletion(object):
         """
 
         complete_content = self.generate()
-        script_src = self._TEMPLATE % {
-            "prop": self.prop,
+        return self._template_source % {
+            "func_name": self.func_name,
+            "prop": self.prog_name,
             "complete_vars": complete_content,
         }
 
-        return script_src
-
     def write_completion(self, complete_src: str) -> bool:
         """Save completion to config path.
 
@@ -125,13 +168,14 @@ class ShellCompletion(object):
             with open(self.inject_path, "r") as f:
                 shell_conf = f.read()
         except Exception as e:
-            raise ShellCompletionError("Read shell config error: {0}".format(e))
+            raise ShellCompletionError(
+                "Read shell config error: {0}".format(e)
+            ) from None
         else:
             _re = re.compile(r"{}[^\s]*".format(full_script_path))
             files = _re.findall(shell_conf)
 
-        injected = True if files else False
-
+        injected = bool(files)
         if injected:
             return False
 
@@ -140,6 +184,6 @@ class ShellCompletion(object):
             with open(self.inject_path, "a") as f:
                 f.write("source %s" % (full_script_path))
         except Exception as e:
-            raise ShellCompletionError(f"Inject error: {str(e)}")
+            raise ShellCompletionError(f"Inject error: {str(e)}") from None
 
         return True
