@@ -1,5 +1,6 @@
 # -*- coding:utf-8 -*-
 
+from typing import Dict
 import os
 import textwrap
 
@@ -10,11 +11,14 @@ class FishCompletion(ShellCompletion):
 
     _SHELL: str = "fish"
 
-    _INJECT_PATH: str = os.environ["HOME"] + "/.config/fish/config.fish"
+    try:
+        _INJECT_PATH: str = os.environ["HOME"] + "/.config/fish/config.fish"
+    except:
+        _INJECT_PATH: str = ""
 
-    _TEMPLATE: str = textwrap.dedent(
+    _template_source: str = textwrap.dedent(
         """\
-        function complete_%(prop)s;
+        function %(func_name)s;
             set -l response;
 
             for value in (env %(complete_vars)s=fish_complete COMP_WORDS=(commandline -cp) \
@@ -36,9 +40,29 @@ class FishCompletion(ShellCompletion):
         end;
 
         complete --no-files --command %(prop)s --arguments \
-        "(complete_%(prop)s)";
+        "(%(func_name)s)";
         """
     )
 
     def generate(self):
-        return " ".join(self.complete_vars.keys())
+        # TODO:improve `fish` completion script.
+        comp_keys = []
+
+        _arguments, _positions, _sub_opts = self._parse(self.complete_vars["args"])
+
+        for x in _arguments:
+            comp_keys.extend(x[0].split())
+
+        sub_q = [_sub_opts]
+        while sub_q:
+            temp: Dict = sub_q.pop(0)
+            for opt_name, p in temp.items():
+                comp_keys.append(opt_name)
+
+                if _a := p.get("_arguments"):
+                    for x in _a:
+                        comp_keys.extend(x[0].split())
+                if _opts := p.get("_sub_opts"):
+                    sub_q.insert(-1, _opts)
+
+        return " ".join(comp_keys)
