@@ -1,41 +1,33 @@
+# -*- coding:utf-8 -*-
 import os
 import pytest
 from pprint import pprint
+from copy import deepcopy
 from .utils import analyze_it
 
-from pigit import Git_Cmds, __project__, Parser
+from pigit.entry import Git_Cmds, argparse_dict
 from pigit.shellcompletion import (
     ZshCompletion,
     BashCompletion,
     FishCompletion,
-    process_argparse,
+    shell_complete,
 )
 
 
 class TestCompletion:
     prog = "pigit"
-    complete_vars = {key: value.get("help", "") for key, value in Git_Cmds.items()}
+    # complete_vars = {key: value.get("help", "") for key, value in Git_Cmds.items()}
+    complete_vars = deepcopy(argparse_dict)
+    cmd_temp = complete_vars["args"]["cmd"]["args"]
+    cmd_temp.update({k: {"help": v["help"], "args": {}} for k, v in Git_Cmds.items()})
     script_dir = os.path.dirname(__file__)
 
     def test_error_complete_vars(self):
         with pytest.raises(TypeError):
             BashCompletion("test", "xxx", ".")
 
-    def test_error_argparse_obj(self):
-        with pytest.raises(TypeError):
-            process_argparse("xxx")
-
-        with pytest.raises(TypeError):
-            process_argparse(object)
-
-    def test_parse_parser(self):
-        p = Parser()
-        p.parse([""])
-        res = process_argparse(p._parser)
-        pprint(res)
-
     def print(self, c):
-        print(c.prop)
+        print(c.prog_name)
         print(c.script_name)
         print(c.inject_path)
 
@@ -46,15 +38,17 @@ class TestCompletion:
 
     @analyze_it
     def test_bash(self):
-        c = BashCompletion(self.prog, self.complete_vars, self.script_dir)
+        c = BashCompletion(None, self.complete_vars, self.script_dir)
         self.print(c)
 
+    @analyze_it
     def test_zsh(self):
-        c = ZshCompletion(self.prog, self.complete_vars, self.script_dir)
+        c = ZshCompletion("pigit-dev", self.complete_vars, self.script_dir)
         self.print(c)
 
-        # print(c.inject_into_shell())
-
     def test_fish(self):
         c = FishCompletion(self.prog, self.complete_vars, self.script_dir)
         self.print(c)
+
+    def test_action(self):
+        shell_complete("bash", "xxx", self.complete_vars, ".", None, "./test.txt")
